import java.io.*;
import java.util.*;
import java.text.SimpleDateFormat;
import java.net.*;
 
import javax.bluetooth.DeviceClass;
import javax.bluetooth.DiscoveryAgent;
import javax.bluetooth.DiscoveryListener;
import javax.bluetooth.LocalDevice;
import javax.bluetooth.RemoteDevice;
import javax.bluetooth.ServiceRecord;
 

public class Cityware implements DiscoveryListener {
   
   
    //object used for waiting
    private Object lock = new Object();
    private RemoteDevice vecDevices[];
    private DiscoveryAgent agent;
    private SimpleDateFormat date;
    private boolean verbose, getnames, offline;
    private String source;
    private String server;
    private String password;
    private myURLencode Encoder;
    private String queue;
    private int queue_size;
    
    
    public static void main(String[] args) {

        Cityware myCityware = new Cityware(args);
    }
    
    
    
    public Cityware(String[] args) {
    	
    	try {
    		queue = "";
    		queue_size = 0;
	   		parseArguements(args);
	   		LocalDevice localDevice = LocalDevice.getLocalDevice();
	        source = localDevice.getBluetoothAddress();
	        vecDevices = new RemoteDevice[128];       
	        agent = localDevice.getDiscoveryAgent();
	        enqueue(localDevice.getBluetoothAddress(), localDevice.getFriendlyName()); //Send info about the scanner
    	} catch (Exception e) {
    		System.err.println("An initialisation error has occured.  Is your Bluetooth enabled?");
    	}
     
  		while(true){    
	        try {
	        	agent.startInquiry(DiscoveryAgent.GIAC, this); 
	            synchronized(lock){
	                lock.wait();
	            }
	        }
	         catch (Exception e) {
  				System.out.println("Cannot start Bluetooth discovery.  Try reseting your Bluetooth dongle.");
  			}

      		for(int i=0; vecDevices[i] != null; i++){
                RemoteDevice remoteDevice = vecDevices[i];
		if(getnames) {
	                String name;
        	        try {
                		name = remoteDevice.getFriendlyName(true);  //winxp: false, osx/linux: true
                		if (name.equals("null") || name.equals(null)){
                			name = "(unknown)";
                		}
               		}
                	catch (Exception e){
                		name = "(unknown)";
                	}
                	enqueue(remoteDevice.getBluetoothAddress(), name);
		}
                vecDevices[i]=null;
            }
       	 	sendData(queue);	
        }       
    }
    	
    	
    	


    
    public void parseArguements(String[] args){
    	
    	String arg;
    	
    	verbose = false;
	getnames = true;
	offline = false;
    	server = "http://cityware.cs.bath.ac.uk/DBAdmin/Log.php";
    	password = "";
    	
    	int i = 0;
    	while (i < args.length && args[i].startsWith("-")) {
            arg = args[i++];

            if (arg.equals("-v")) {
                verbose = true;
            }
            if (arg.equals("-f")) {
                offline = true;
            }
            if (arg.equals("-n")) {
                getnames = false;
            }
            if (arg.equals("-h")) {
            	System.out.println("");
            	System.out.println("");
                System.out.println("This is the Cityware utility that allows you to scan for nearby devices and store the data locally.");
                System.out.println("Developed by Vassilis Kostakos, University of Bath, 2008.");
                System.out.println("");
                System.out.println("Usage:");
                System.out.println("");
                System.out.println("-h             : Prints this message.");
 				System.out.println("-p <password>  : Set the access password.  You can obtain it from http://apps.facebook.com/cityware");
 				System.out.println("-s <server>    : Specifies a Cityware server.  If not set, the default value is used.");
 				System.out.println("-v             : Verbose mode.  Prints debugging information.");
 				System.out.println("-n             : No names.  Does not inquire device names.");
 				System.out.println("-f             : Offline.  Do not attempt to upload data.");
 				System.out.println("");	
 				System.exit(0);
            }
            else if (arg.equals("-s")) {
                if (i < args.length){
                    server = args[i++];
                }
                else{
                    System.err.println("-s requires a parameter.  Reverting to default.");	
                }	
            }
            else if (arg.equals("-p")) {
                if (i < args.length){
                    password = args[i++];
                }
                else{
                    System.err.println("-p requires a password.  Stopping...");		
                    System.exit(1);
                }
            }
        }
       	while (password == "" && offline == false) {
       		try {
	           	BufferedReader stdin = new BufferedReader( new InputStreamReader(System.in));
	           	// Prompt the user
	            System.out.print( "Please enter your password: " );
	            password = stdin.readLine();
       		}
	        catch (Exception e) {System.exit(1);}
         }
    }
    
    /**
     * This call back method will be called for each discovered bluetooth device.
     */
    public void deviceDiscovered(RemoteDevice btDevice, DeviceClass cod) {
       
        //sendData(btDevice.getBluetoothAddress(),cod.getMajorDeviceClass()+"-"+cod.getMinorDeviceClass(),"");
		enqueue(btDevice.getBluetoothAddress(),"");
        int i = 0;
        while(vecDevices[i] != null)
        	{i++;}
        vecDevices[i] = btDevice;
    }
 

    
    
    public void enqueue(String address, String name){
    	
    	queue_size++;
    	String date = new SimpleDateFormat("yyyy-MM-dd").format(new Date());
    	String time = new SimpleDateFormat("HH:mm:ss").format(new Date());
    	System.out.println(date+","+time+","+address+","+name);
 		String s =  "&Data"+queue_size+"=" + address  + "&Name"+queue_size+"=" + Encoder.encode(name) + 
 					"&Date"+queue_size+"=" + date + "&Time"+queue_size+"=" + time + "&Context_type"+queue_size+"=101";
    	queue = queue + s;
     }
     
     
    
    public void sendData(String data){  
	
		String reply,line;
		boolean fail = false; 

		if(offline){
	      		queue="";
	      		queue_size=0;
			if(verbose){
      				System.out.println("In offline mode, not uploading");
			}
			return;
		}
			
      	try {
      		String date_now = new SimpleDateFormat("yyyy-MM-dd").format(new Date());
    		String time_now = new SimpleDateFormat("HH:mm:ss").format(new Date());
    		String now = "&Date_now="+date_now + "&Time_now="+time_now;
      		String s = "Source=" + source + "&p=" + password + now + data;
			//write data
         	URL url = new URL(server);
         	URLConnection conn = url.openConnection();
         	conn.setDoOutput(true);
           	OutputStreamWriter out = new OutputStreamWriter(conn.getOutputStream());
         	out.write(s);
         	out.close();
         	//read response         
       		BufferedReader in = new BufferedReader(new InputStreamReader(conn.getInputStream()));
         	reply="";
         	while ((line = in.readLine()) != null)
         		reply += line;
         	if(verbose){
      			System.out.println(s);
      			System.out.println("Response - "+reply);
         	} else if(reply.toLowerCase().indexOf("error") >-1){
         		System.err.println("The server reported an error - "+reply);
         	}
         	in.close(); 
      } catch (MalformedURLException mue) {
         System.err.println("Could not send the data.  Check that the server URL is valid."); 
         fail = true;
      } catch (IOException ioe) {
         System.err.println("Could not send the data: no connection available, or the the server is down. Buffering ("+queue_size+")"); 
         fail = true;
      } catch (Exception e) {
      	 System.err.println("An error has occured.");
      	 e.printStackTrace();
      	 fail = true;
	  } 
      finally {
      	if(!fail){
	      	queue="";
	      	queue_size=0;
      	}	
      }	   	
    }
    
    
    
    
    
    //no need to implement this method since services are not being discovered
    public void servicesDiscovered(int transID, ServiceRecord[] servRecord) {}
 
    //no need to implement this method since services are not being discovered
    public void serviceSearchCompleted(int transID, int respCode) {}
   
    /**
     * This callback method will be called when the device discovery is completed.
     */
    public void inquiryCompleted(int discType) {
	
		synchronized(lock){
            lock.notify();
        }  
        switch (discType) {
            case DiscoveryListener.INQUIRY_COMPLETED :
                break;
               
            case DiscoveryListener.INQUIRY_TERMINATED :
                System.err.println("An error has occured: INQUIRY_TERMINATED.");
                break;
               
            case DiscoveryListener.INQUIRY_ERROR :
                System.err.println("An error has occured:  INQUIRY_ERROR.");
                break;
 
            default :
                System.err.println("An error has occured: Unknown Response Code.");
                break;
        }
    }
    
    

    
}